#!/bin/sh
#
# MXplot -- a helper application for MX
# Author:  James F. Fait
# Version 1.0
# Copyright October 2000, The University of Georgia Research Foundation, Inc.
# Copyright Assigned to James F. Fait, Ph. D.
#
# Version 1.1
#   A few modifications made by William Lavender (October 2002):
#   1.  Use 'package require BLT' rather than bltwish.
#   2.  If evaluation of the plot arguments fail, set the result to 0.
#   3.  The 'bltGraph.tcl' file can be found in the directory $blt_library.
#
# See file "LICENSE.mxplot" for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# Look for wish in the users' PATH. \
exec wish "$0" ${1+"$@"}

package require BLT

proc static {args} {
 set procName [lindex [info level -1] 0]
   foreach varName $args {
     uplevel 1 "upvar #0 staticvars($procName:$varName) $varName"
   }
}
proc read_file {filename i } {
  global x_0
  global x_1
  global x_2
  global x_3
  global x_4
  global x_5
  global x_6
  global x_7
  global x_8
  global x_9
  global x_10
  global y_0
  global y_1
  global y_10
  global y_2
  global y_3
  global y_4
  global y_5
  global y_6
  global y_7
  global y_8
  global y_9
  set y_name y_$i
  set x_name x_$i
  set FH [open $filename r]
  while { [expr ! [eof $FH ]] } {
    gets $FH data
    if { [eof $FH] } { break }
#    puts $data
    set data  [ string trim $data ]
    set s_end [ string first " " $data ]
    set xval [ string range $data 0 $s_end ]
    puts $xval
    set xval  [ string trim $xval ]
    switch $xval {
        SFF {
        }
        beamline_name {
        }
        edge_energy {
        }
        sff_header1 {
        }
        sff_header2 {
        }
        %devices  {
          puts $xval
        }
        default {
           set res [expr double($xval)]
           set data  [ string range $data $s_end end ]
           set data  [ string trim $data ]
           [ set x_name ] append $res
           set s_end [ string first " " $data ]
#           puts $s_end
           if { $s_end != -1 } {
           set xval [ string range $data 0 $s_end ]
           } else {
           set xval $data
           }
#           puts $xval
           set res [expr double($xval)]
           [ set y_name ] append $res
          }
       }
  }
  close $FH
}
# routine to read stdin and output result to stdout
proc ReadStdin {} {
  global init
  global x_0
  global x_1
  global x_2
  global x_3
  global x_4
  global x_5
  global x_6
  global x_7
  global x_8
  global x_9
  global x_10
  global y_0
  global y_1
  global y_2
  global y_3
  global y_4
  global y_5
  global y_6
  global y_7
  global y_8
  global y_9
  global y_10
  global title
  global xlabel
  global ylabel
  global .g1
  global onexit
  global axis_color
  static count
  static num_independent innermost_independent plot_args
  static v_list
  static used
  if {$init } {
      set init false
      set axis_color(0) red 
      set axis_color(1) green 
      set axis_color(2) blue 
      set axis_color(3) magenta3 
      set axis_color(4) black 
      set axis_color(5) purple 
      set axis_color(6) pink 
      set axis_color(7) yellow 
      set axis_color(8) orange 
      set axis_color(9) red3
      set axis_color(10) blue3
      vector x_0
      vector x_1
      vector x_2
      vector x_3
      vector x_4
      vector x_5
      vector x_6
      vector x_7
      vector x_8
      vector x_9
      vector x_10
      vector y_0
      vector y_1
      vector y_2
      vector y_3
      vector y_4
      vector y_5
      vector y_6
      vector y_7
      vector y_8
      vector y_9
      vector y_10
      x_0 notify always
      x_10 notify always
      x_1 notify always
      x_2 notify always
      x_3 notify always
      x_4 notify always
      x_5 notify always
      x_6 notify always
      x_7 notify always
      x_8 notify always
      x_9 notify always
      y_0 notify always
      y_10 notify always
      y_1 notify always
      y_2 notify always
      y_3 notify always
      y_4 notify always
      y_5 notify always
      y_6 notify always
      y_7 notify always
      y_8 notify always
      y_9 notify always
      set used "-1"
  }
  if { [eof stdin] } then {
      fileevent stdin readable ""
       if { $onexit == "1"} then {
#         puts "Unregister [clock clicks]"
#         exit
       }
#     exit
  }
  gets stdin st
#  puts $st
  regsub -all "\[\[\]" $st {(} result
  regsub -all "\[\]\]" $result {)} result
  regsub -all "\[\;\]" $result { } st
  set line_list [ split $st ]
  set key_word [ lindex $line_list 0 ]
#  puts "-$key_word-"
  switch $key_word {
    start_plot {  
      set plot_args [ lrange $line_list 3 end ]
      set num_independent [ lindex $line_list 1 ]
      set innermost_independent [ lindex $line_list 2 ]
      if { $innermost_independent < 0 } { set innermost_independent 0 }
      set used "-1"
      set count "0"
    }
    set        {  
      set q1 [ string first {'} $st ]
      set q2 [ string last {'} $st ]
#      set [set [ lindex $line_list 1 ]] [string range $st $q1 $q2 ] 
      set s_key [ lindex $line_list 1 ]
      switch $s_key {
         title  {
            .g1 configure -title [string range $st $q1 $q2 ]
         }
         xlabel {
            set q1 [expr $q1 + 1]
            set q2 [expr $q2 - 1]
            .g1 axis configure x -title [string range $st $q1 $q2 ]
         }
         marker {
#  marker for peak position
# marker needs to come after plot and replot for correct scaling
# format:  set marker $xval 
            set xval [lindex $line_list 2 ]
            set y2 [expr $y_0(max) * 2 ]
            set y1 [expr $y_0(min) - $y2 ]
            .g1 marker create line -coords { $xval $y1 $xval $y2 } 
            .g1 marker configure -outline $axis_color(2)
         }
         terminal {
         }
         nokey    {
         }
       }
    }
    data       {  
      set data [ string range $st 5 end ]
      set data [ string trimleft $data]
# get the independent variable(s)
      for { set i 0 } { $i < $num_independent } { incr i } {
        set s_end [ string first " " $data ]
#puts "datas- $data"
#puts "s_end- $s_end"
        set x_vals($i) [expr double([ string range $data 0 $s_end ])]
        set data  [ string range $data $s_end end ]
        set data  [ string trim $data ]
        x_0 append $x_vals($innermost_independent)
      }
      set i 0
      while { [ set s_end [ string first " " $data]] > 0 } {
        set f($i) [expr double([ string range $data 0 $s_end ])]
        incr i
        set data  [ string range $data $s_end end ]
        set data  [ string trim $data ]
      }
      set f($i) $data
      incr i
      set num_dependent $i
# debug output
#      for { set i 0 } { $i < $num_independent } { incr i } {
#        puts -nonewline $x_vals($i)
#      }
#      for { set i 0 } { $i < $num_dependent } { incr i } {
#        puts -nonewline $f($i)
#      }
#      puts " " 
# evaluate plot arguments
      for { set i 0 } { $i < [llength $plot_args] } { incr i } {
        set status [ catch {set res [expr double([lindex $plot_args $i])]} ]
        if { $status != 0 } {
           set res 0
        }
# create the vector name
        set v_name y_$i
        if { $i > $used } { 
#           puts "$v_name"
           set used $i
           .g1 element create $v_name -label [lindex $plot_args $i] \
              -ydata [set v_name] -xdata x_0 -symbol splus -color $axis_color($i) \
              -pixels 5
        }
        [ set v_name ] append $res
#        puts -nonewline $res
#        puts -nonewline " "
      }
#      puts " " 
#      puts "update after data " 
#      update idletasks
    }
    replot -
    plot       {  
        set sp [string first { } $st] 
        if { $sp >= 0 } {
          set res [string range $st $sp end]
        } else {
          set res {}
        }
        set filename {}
# (re)plot can be empty, have an equation, or have a file name
# deal with the file first (inside 'single quotes')
     if { [string first {'} $st ] != -1 } {
       set q1 [ string first {'} $st ]
       set q2 [ string last {'} $st ]
       set q1 [expr $q1 + 1]
       set q2 [expr $q2 - 1]
       set filename [string range $st $q1 $q2 ]
#       puts ": $filename"
# process file, placing in x_0 and y_0
       set used [expr $used + 1]
       set x_name x_$used
       set v_name y_$used
       read_file $filename $used
       .g1 element create [set v_name]  \
           -ydata [set v_name] -xdata [set x_name] -symbol splus -color \
           $axis_color($used) -pixels 5 -linewidth 0
# spawn perl script with equation, feed it x values, and read back x,y pairs
     } elseif {[string length [string trim $res ]] > 0 } {
#       puts ": $res"
       regsub -all {\(x} $res {( $x} res
#       puts ": $res"
       set filename "tmp[pid].plt"
#       puts ": $filename"
       set PIPE [ open "| function.pl | tee $filename " w+]
       set used [expr $used + 1]
       set x_name x_$used
       set v_name y_$used
       x_0 populate [set x_name] 10
       puts $PIPE $res
       for {set i 0} {$i < [ [set x_name] length ] } { incr i } {
#         puts " $i $x_1($i)"
         puts $PIPE $x_1($i)
       }
       flush $PIPE
       catch { close $PIPE }
# process file, placing in x_1 and y_1
       read_file $filename $used
       .g1 element create [set v_name]\
           -ydata [set v_name] -xdata [set x_name] -symbol {}  \
           -color $axis_color($used) -pixels 5
       file delete $filename
     }

    }
    exit       {  
       if {$onexit == "1"} {
         exit
       }
#      puts Exiting
    }
    "" {
#      puts "empty string " 
    }
    default    {  
      puts "Unknown Plot Command"
    }
  }
# update idletasks
# puts  "update screen [clock clicks]"
# incr count
}
  
# start of main routine
# list of vectors, with names created on the fly of the form y_1, y_2, ...

if { [info commands "namespace"] == "namespace" } {
    if { $tcl_version >= 8.0 } {
        namespace import blt::*
    } else {
        catch { import add blt }
    }
    if { $tcl_version >= 8.0 } {
        namespace import -force blt::tile::*
    } else {
        import add blt::tile
    }
} else {
    foreach cmd { button checkbutton radiobutton frame label
        scrollbar toplevel menubutton listbox } {
        if { [info command tile${cmd}] == "tile${cmd}" } {
            rename ${cmd} ""
            rename tile${cmd} ${cmd}
        }
    }
}

set onexit "1"
foreach arg $argv {
#  puts $arg
   switch $arg {
     exit {
       set onexit "0"
     }
   }
}



array set cursors {
    w left_side
    e right_side
    n top_side
    s bottom_side
    sw bottom_left_corner
    ne top_right_corner
    se bottom_right_corner
    nw top_left_corner
}


array set pageInfo {
    gripSize 8
    scale 0.25
    radioFont -*-helvetica-medium-r-*-*-11-120-*-*-*-*-*-*
    labelFont -*-helvetica-bold-r-*-*-12-120-*-*-*-*-*-*
    printCmd	"lpr "
    printFile   "out.ps"
    -landscape  "1" 
    plotSize    "maxpect"
    printTo "printCmd"
}


proc SetUnits { units }  {
    global pageInfo
    switch -glob $units {
	"i*"    { set pageInfo(uscale) [winfo fpixels . 1i]  }
	"c*"    { set pageInfo(uscale) [winfo fpixels . 1c] }
	default { error "unknown unit \"$units\"" }
    }
    set pageInfo(units) [string index $units 0]
}


proc ConvertUnits { value } {
    global pageInfo
    set value [expr double($value) / $pageInfo(uscale)]
    return [format "%.1f%s" $value $pageInfo(units)]
}


proc SetPaperSize { unit } {
    global pageInfo
    SetUnits $unit
    set pageInfo(-paperwidth) [lindex $pageInfo(paperSize) 0]
    set pageInfo(-paperheight) [lindex $pageInfo(paperSize) 1]
    ApplyPs
}
    
proc SetCanvasSize { canvas width height } {
    global pageInfo

    set width [winfo pixels . $width]
    set height [winfo pixels . $height]
    $canvas configure -width $width -height $height
}

proc SetCanvasOrientation { canvas } {
    global pageInfo
    set width $pageInfo(paperWidth)
    set height $pageInfo(paperHeight)
    SetCanvasSize $canvas $width $height
}


proc GetPsOptions { graph } {
    global pageInfo

    foreach opt [$graph postscript configure] {
	set pageInfo([lindex $opt 0]) [lindex $opt 4]
    }
}

proc SetOutline { canvas } {
    global pageInfo
    foreach var { gripSize xMin yMin xMax yMax } {
	set $var $pageInfo($var)
    }
    set xMid [expr ($xMax + $xMin - $gripSize) * 0.5]
    set yMid [expr ($yMax + $yMin - $gripSize) * 0.5]
    $canvas coords image $xMin $yMin 
    $canvas itemconfigure image \
	-width [expr $xMax - $xMin] -height [expr $yMax - $yMin]
    $canvas coords nw \
	$xMin $yMin [expr $xMin + $gripSize] [expr $yMin + $gripSize] 
    $canvas coords se \
	[expr $xMax - $gripSize] [expr $yMax - $gripSize] $xMax $yMax 
    $canvas coords ne \
	[expr $xMax - $gripSize] [expr $yMin + $gripSize] $xMax $yMin 
    $canvas coords sw \
	$xMin $yMax [expr $xMin + $gripSize] [expr $yMax - $gripSize] 
    SetCanvasOrientation $canvas
    $canvas coords n \
	$xMid $yMin [expr $xMid + $gripSize] [expr $yMin + $gripSize] 
    $canvas coords s \
	$xMid [expr $yMax - $gripSize] [expr $xMid + $gripSize] $yMax
     $canvas coords e \
 	[expr $xMax - $gripSize] $yMid $xMax [expr $yMid + $gripSize] 
     $canvas coords w \
 	$xMin $yMid [expr $xMin + $gripSize] [expr $yMid + $gripSize] 
}

proc CreateOutline { canvas } {
    global pageInfo
    foreach var { gripSize xMin yMin xMax yMax } {
	set $var $pageInfo($var)
    }
    if { ![bitmap exists pattern8] } {
	bitmap define pattern8 { {8 8} {ff 00 ff 00 ff 00 ff 00 } }
    }
    $canvas create eps $xMin $yMin \
	-tags "outline image" \
	-width [expr $xMax - $xMin] \
	-height [expr $yMax - $yMin]
    $canvas bind image <ButtonPress-1>   "StartMove $canvas %x %y"
    $canvas bind image <B1-Motion>       "MoveOutline $canvas %x %y"
    $canvas bind image <ButtonRelease-1> "EndMove $canvas"

    $canvas bind image <Shift-B1-Motion> "ConstrainMoveOutline $canvas %x %y"
    $canvas bind image <Enter> "EnterImage $canvas"
    $canvas bind image <Leave> "LeaveImage $canvas"
    focus $canvas
    $canvas create rectangle \
	$xMin $yMin [expr $xMin + $gripSize] [expr $yMin + $gripSize] \
	-tags "outline grip nw" 
    $canvas create rectangle \
	[expr $xMax - $gripSize] [expr $yMax - $gripSize] $xMax $yMax \
	-tags "outline grip se" 
    $canvas create rectangle \
	[expr $xMax - $gripSize] [expr $yMin + $gripSize] $xMax $yMin \
	-tags "outline grip ne" 
    $canvas create rectangle \
	$xMin $yMax [expr $xMin + $gripSize] [expr $yMax - $gripSize] \
	-tags "outline grip sw" 

    set xMid [expr ($xMax + $xMin - $gripSize) * 0.5]
    set yMid [expr ($yMax + $yMin - $gripSize) * 0.5]
    $canvas create rectangle \
	$xMid $yMin [expr $xMid + $gripSize] [expr $yMin + $gripSize] \
	-tags "outline grip n" 
    $canvas create rectangle \
	$xMid [expr $yMax - $gripSize] [expr $xMid + $gripSize] $yMax \
	-tags "outline grip s" 
     $canvas create rectangle \
 	[expr $xMax - $gripSize] $yMid $xMax [expr $yMid + $gripSize] \
 	-tags "outline grip e" 
     $canvas create rectangle \
 	$xMin $yMid [expr $xMin + $gripSize] [expr $yMid + $gripSize] \
 	-tags "outline grip w" 
    foreach grip { e w s n sw ne se nw } {
	$canvas bind $grip <ButtonPress-1>   "StartResize %W $grip %x %y"
	$canvas bind $grip <B1-Motion>	     "ResizeOutline %W %x %y"
	$canvas bind $grip <ButtonRelease-1> "EndResize %W $grip %x %y"
	$canvas bind $grip <Enter>	     "EnterGrip %W $grip %x %y"
	$canvas bind $grip <Leave>	     "LeaveGrip %W $grip"
    }
    $canvas raise grip
    $canvas itemconfigure grip -fill red -outline black

    set pageInfo(image) [image create photo]
    $pageInfo(graph) snap $pageInfo(image)
    $canvas itemconfigure image -image $pageInfo(image)
}


proc EnterImage  { canvas } {
    global cursors
    global pageInfo
    bind $canvas <KeyPress-Left>  {
	MoveOutline %W [expr $pageInfo(lastX) - 1]  $pageInfo(lastY)
    }
    bind $canvas <KeyPress-Right> {
	MoveOutline %W [expr $pageInfo(lastX) + 1]  $pageInfo(lastY)
    }
    bind $canvas <KeyPress-Up>   {
	MoveOutline %W $pageInfo(lastX) [expr $pageInfo(lastY) - 1]
    }
    bind $canvas <KeyPress-Down> {
	MoveOutline %W $pageInfo(lastX) [expr $pageInfo(lastY) + 1]
    }
    focus $canvas
    $canvas configure -cursor fleur
    set pageInfo(lastX) 0
    set pageInfo(lastY) 0
}


proc LeaveImage { canvas } {    
    bind $canvas <KeyPress-Left> ""
    bind $canvas <KeyPress-Right> ""
    bind $canvas <KeyPress-Up> ""
    bind $canvas <KeyPress-Down> ""
    $canvas configure -cursor ""
}

proc EnterGrip  { canvas grip x y } {
    global pageInfo
    $canvas itemconfigure $grip -fill blue -outline black
    set pageInfo(grip) $grip
    global cursors
    bind $canvas <KeyPress-Left>  {
	ResizeOutline %W [expr $pageInfo(lastX) - 1] $pageInfo(lastY)
    }
    bind $canvas <KeyPress-Right> {
	ResizeOutline %W [expr $pageInfo(lastX) + 1] $pageInfo(lastY)
    }
    bind $canvas <KeyPress-Up> {
	ResizeOutline %W $pageInfo(lastX) [expr $pageInfo(lastY) - 1] 
    }
    bind $canvas <KeyPress-Down> {
	ResizeOutline %W $pageInfo(lastX) [expr $pageInfo(lastY) + 1] 
    }
    focus $canvas
    $canvas configure -cursor $cursors($grip)
    set pageInfo(lastX) $x
    set pageInfo(lastY) $y
}

proc LeaveGrip { canvas grip } {    
    $canvas itemconfigure $grip -fill red -outline black
    bind $canvas <KeyPress-Left> ""
    bind $canvas <KeyPress-Right> ""
    bind $canvas <KeyPress-Up> ""
    bind $canvas <KeyPress-Down> ""
    $canvas configure -cursor ""
}

proc StartMove { canvas x y } {
    global pageInfo 
    set pageInfo(lastX) $x
    set pageInfo(lastY) $y
    set pageInfo(direction) "undecided"
    $canvas configure -cursor fleur
}

proc MoveOutline { canvas x y } {
    global pageInfo
    $canvas move outline [expr $x - $pageInfo(lastX)] [expr $y - $pageInfo(lastY)]
    set pageInfo(lastX) $x
    set pageInfo(lastY) $y
}

proc ConstrainMoveOutline { canvas x y } {
    global pageInfo

    set dx [expr $x - $pageInfo(lastX)]
    set dy [expr $y - $pageInfo(lastY)]

    if { $pageInfo(direction) == "undecided" } {
	if { abs($dx) > abs($dy) } {
	    set pageInfo(direction) x
	    $canvas configure -cursor sb_h_double_arrow
	} else {
	    set pageInfo(direction) y
	    $canvas configure -cursor sb_v_double_arrow
	}
    }
    switch $pageInfo(direction) {
	x { set dy 0 ; set pageInfo(lastX) $x } 
	y { set dx 0 ; set pageInfo(lastY) $y }
    }
    $canvas move outline $dx $dy
}

proc EndMove { canvas } {
    $canvas configure -cursor ""

    set coords [$canvas coords image]
    set x [lindex $coords 0]
    set y [lindex $coords 1]
    set w [$canvas itemcget image -width]
    set h [$canvas itemcget image -height]

    global pageInfo
    set pageInfo(xMin) $x
    set pageInfo(xMin) $y
    set pageInfo(xMax) [expr $x + $w]
    set pageInfo(yMax) [expr $y + $h]

    global pageInfo
    set pageInfo(-padx) [list $pageInfo(xMin) [expr $pageInfo(paperWidth) - $pageInfo(xMax)]]
    set pageInfo(-pady) [list $pageInfo(yMin) [expr $pageInfo(paperHeight) - $pageInfo(yMax)]]
}

proc StartResize { canvas grip x y } {
    global pageInfo
    set pageInfo(grip) $grip
    $canvas itemconfigure $grip -fill red -outline black
    $canvas raise grip
    global cursors
    $canvas configure -cursor $cursors($grip)
    set pageInfo(lastX) $x
    set pageInfo(lastY) $y
}

proc EndResize { canvas grip x y } {
    ResizeOutline $canvas $grip $x $y
    $canvas itemconfigure $grip -fill "" -outline ""
    $canvas configure -cursor ""
}

proc ResizeOutline { canvas x y } {
    global pageInfo

    foreach var { gripSize xMin yMin xMax yMax } {
	set $var $pageInfo($var)
    }
    switch $pageInfo(grip) {
	n {
	    set yMin $y
	}
	s {
	    set yMax $y
	}
	e {
	    set xMax $x
	}
	w {
	    set xMin $x
	}
	sw {
	    set xMin $x ; set yMax $y
	}
	ne {
	    set xMax $x ; set yMin $y
	}
	se {
	    set xMax $x ; set yMax $y
	}
	nw {
	    set xMin $x ; set yMin $y
	}
    }
    set width [expr $xMax - $xMin]
    set height [expr $yMax - $yMin]
    if { ($width < 1) || ($height < 1) } {
	return
    }
    SetOutline $canvas
    foreach var { xMin yMin xMax yMax } {
	set pageInfo($var) [set $var]
    }
}

proc ComputePlotGeometry { graph } {
    global pageInfo

    GetPsOptions $graph
    set width [winfo width $graph]
    set height [winfo height $graph]
    if { $pageInfo(-width) > 0 } {
	set width $pageInfo(-width)
    }
    if { $pageInfo(-height) > 0 } {
	set height $pageInfo(-height)
    }

    set left [lindex $pageInfo(-padx) 0]
    set right [lindex $pageInfo(-padx) 1]
    set top [lindex $pageInfo(-pady) 0]
    set bottom [lindex $pageInfo(-pady) 1]
    set padx [expr $left + $right]
    set pady [expr $top + $bottom]

    if { $pageInfo(-paperwidth) > 0 } {
	set paperWidth $pageInfo(-paperwidth)
    } else {
	set paperWidth [expr $width + $padx]
    }
    if { $pageInfo(-paperheight) > 0 } {
	set paperHeight $pageInfo(-paperheight)
    } else {
	set paperHeight [expr $height + $pady]
    }
    if { $pageInfo(-landscape) } {
	set temp $paperWidth
	set paperWidth $paperHeight
	set paperHeight $temp
    }

    set scale 1.0
    if { $pageInfo(-maxpect) } {
	set xScale [expr ($paperWidth - $padx) / double($width)]
	set yScale [expr ($paperHeight - $pady) / double($height)]
	set scale [expr min($xScale,$yScale)]
	set bboxWidth [expr round($width * $scale)]
	set bboxHeight [expr round($height * $scale)]
    } else {
	if { ($width + $padx) > $paperWidth } {
	    set width [expr $paperWidth - $padx]
	}
	if { ($height + $pady) > $paperHeight } {
	    set height [expr $paperHeight - $pady]
	}
	set bboxWidth $width
	set bboxHeight $height
    }
    set x $left
    set y $top
    if { $pageInfo(-center) } {
	if { $paperWidth > $bboxWidth }  {
	    set x [expr ($paperWidth - $bboxWidth) / 2]
	}
	if { $paperHeight > $bboxHeight } {
	    set y [expr ($paperHeight - $bboxHeight) / 2]
	}
    }
    set pageInfo(xMin) [expr $x * $pageInfo(scale)]
    set pageInfo(yMin) [expr $y * $pageInfo(scale)]
    set pageInfo(xMax) [expr ($x + $bboxWidth) * $pageInfo(scale)]
    set pageInfo(yMax) [expr ($y + $bboxHeight) * $pageInfo(scale)]
    set pageInfo(paperHeight) [expr $paperHeight * $pageInfo(scale)]
    set pageInfo(paperWidth) [expr $paperWidth * $pageInfo(scale)]
}

proc PsDialog { graph } {
    global pageInfo

    set pageInfo(graph) $graph
    set top $graph.top
    toplevel $top

    catch {$graph postscript configure  -landscape $pageInfo(-landscape)}
    option add *graph.top*Radiobutton.font $pageInfo(radioFont)
    GetPsOptions $graph
    ComputePlotGeometry $graph
    set canvas $top.layout
    canvas $canvas -confine yes \
	-width $pageInfo(paperWidth) -height $pageInfo(paperHeight) -bg gray \
	-bd 2 -relief sunken 
    CreateOutline $canvas
    SetCanvasOrientation $canvas
    label $top.titleLabel -text "PostScript Options"
    table $top \
	0,0 $top.titleLabel -cspan 7 \
	1,0 $canvas -cspan 7

    set row 2
    set col 0
    label $top.paperLabel -text "Paper"
    radiobutton $top.letter -text "Letter 8 1/2 x 11 in." -value "8.5i 11i" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize i"
    radiobutton $top.a3 -text "A3 29.7 x 42 cm." -value "28.7c 41c" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize c"
    radiobutton $top.a4 -text "A4 21 x 29.7 cm." -value "21c 29.7c" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize c"
    radiobutton $top.a5 -text "A5 14.85 x 21 cm." -value "14.85c 21c" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize c"
    radiobutton $top.legal -text "Legal 8 1/2 x 14 in." -value "8.5i 14i" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize i"
    radiobutton $top.large -text "Large 11 x 17 in." -value "11i 17i" \
	-variable pageInfo(paperSize) \
	-command "SetPaperSize i"
    table configure $top r$row -pady { 4 0 }
    table $top \
	$row,$col     $top.paperLabel -anchor e \
	$row+0,$col+1 $top.letter -anchor w \
	$row+1,$col+1 $top.legal -anchor w \
	$row+2,$col+1 $top.large -anchor w \
	$row+0,$col+2 $top.a3 -anchor w \
	$row+1,$col+2 $top.a4 -anchor w \
	$row+2,$col+2 $top.a5 -anchor w 

    incr row 3

    label $top.orientLabel -text "Orientation"
    radiobutton $top.portrait -text "Portrait" -value "0" \
	-variable pageInfo(-landscape) -command "ApplyPs"
    radiobutton $top.landscape -text "Landscape" -value "1" \
	-variable pageInfo(-landscape) -command "ApplyPs"
    table configure $top r$row -pady { 4 0 }
    table $top \
	$row,$col+0   $top.orientLabel -anchor e \
	$row,$col+1 $top.portrait -anchor w \
	$row,$col+2 $top.landscape -anchor w 

    incr row 6

    set col 0
    label $top.plotLabel -text "Plot Options"
    table $top \
	$row,$col   $top.plotLabel -cspan 3 
    incr row
    label $top.sizeLabel -text "Size"
    radiobutton $top.default -text "Default" -value "default" \
	-variable pageInfo(plotSize) \
 	-command "SetPlotSize"
    radiobutton $top.maxpect -text "Max Aspect" -value "maxpect" \
 	-variable pageInfo(plotSize) \
 	-command "SetPlotSize"
    radiobutton $top.resize -text "Resize" -value "resize" \
 	-variable pageInfo(plotSize) \
 	-command "SizeDialog $graph {Adjust Plot Size}"
    table configure $top r$row -pady { 4 0 }
    table $top \
	$row,$col   $top.sizeLabel -anchor e \
	$row,$col+1 $top.default -anchor w \
	$row+1,$col+1 $top.maxpect -anchor w \
	$row+2,$col+1 $top.resize -anchor w 

    #incr row 4
    
    set pageInfo(oldPadX) $pageInfo(-padx)
    set pageInfo(oldPadY) $pageInfo(-pady)

    label $top.posLabel -text "Position"
    set pageInfo(position) $pageInfo(-center)
    radiobutton $top.center -text "Center" -value "1" \
	-variable pageInfo(position) -command {
	    set pageInfo(-center) 1
	    CenterPlot
	}
    radiobutton $top.origin -text "Origin" -value "0" \
	-variable pageInfo(position) -command {
	    set pageInfo(-center) 0
	    ApplyPs
	}
    radiobutton $top.move -text "Move" -value "move" \
 	-variable pageInfo(position) -command {
	    set pageInfo(-center) 0
	    MoveDialog
	}
    table configure $top r$row -pady { 4 0 }
    table $top \
	$row,$col+2 $top.posLabel -anchor e \
	$row,$col+3 $top.center -anchor w \
	$row+1,$col+3 $top.origin -anchor w \
	$row+2,$col+3 $top.move -anchor w 

    incr row 4
    label $top.printLabel -text "Print To"
    radiobutton $top.toCmd -text "Command" -value "printCmd" \
	-variable pageInfo(printTo) -command "
	    $top.fileEntry configure -textvariable pageInfo(printCmd)
	"
    radiobutton $top.toFile -text "File" -value "printFile" \
	-variable pageInfo(printTo) -command "
            $top.fileEntry configure -textvariable pageInfo(printFile)
	"
    entry $top.fileEntry 
    set rt $row
    incr rt
    table configure $top r$row -pady { 4 0 }
    table configure $top r$rt -pady { 4 0 }
    incr rt
    table configure $top r$rt -pady { 4 0 }
    table $top \
	$row,0   $top.printLabel -anchor e \
	$row,1 $top.toCmd -anchor w \
	$row+1,1   $top.toFile -anchor w \
	$row+2,1 $top.fileEntry -anchor w -fill x -cspan 3 
    $top.toCmd invoke
    incr row 3
    #table configure $top c4 -width .125i
    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.print -text "Done" -command "PrintPs $graph $top"
    button $top.advanced -text "Options" -command "MarginDialog $graph"
    table $top \
	$row,1 $top.print  -width 1i -pady 2  \
	$row,2 $top.advanced  -width 1i -pady 2 \
	$row,3 $top.cancel  -width 1i -pady 2 -anchor w 

    SetUnits "inches"
    foreach label [info commands $top.*Label] {
	$label configure -font $pageInfo(labelFont) -padx 4
    }
}

proc PrintPs { graph top } {
    global pageInfo
    $graph postscript output "out.ps"
    puts stdout "wrote file \"out.ps\"."
    puts stdout $pageInfo(printTo)
    switch $pageInfo(printTo) {
     "printCmd" {
      set str "$pageInfo(printCmd) out.ps"
      puts $str
      puts stdout [eval exec $str]
     }
    }
    flush stdout
    destroy $top
}

proc ApplyPs { } {
    global pageInfo

    set graph $pageInfo(graph)
    foreach option [$graph postscript configure] {
	set var [lindex $option 0]
	set old [lindex $option 4]
	if { [catch {$graph postscript configure $var $pageInfo($var)}] != 0 } {
	    $graph postscript configure $var $old
	    set pageInfo($var) $old
	}
    }
    ComputePlotGeometry $graph
    foreach var { -paperheight -paperwidth -width -height } {
	set pageInfo($var) [ConvertUnits $pageInfo($var)]
    }
    SetOutline $graph.top.layout
}

proc StartChange { w delta } {
    ChangeSize $w $delta
    global pageInfo
    set pageInfo(afterId) [after 300 RepeatChange $w $delta]
}

proc RepeatChange { w delta } {
    ChangeSize $w $delta
    global pageInfo
    set pageInfo(afterId) [after 100 RepeatChange $w $delta]
}

proc EndChange { w } {
    global pageInfo
    after cancel $pageInfo(afterId)
}

proc ChangeSize { w delta } {
    set f [winfo parent $w]
    set value [$f.entry get]
    set value [expr $value + $delta]
    if { $value < 0 } {
	set value 1
    }
    $f.entry delete 0 end
    $f.entry insert 0 $value
}

proc MakeSizeAdjustor { w label var } {
    frame $w
    label $w.label -text $label
    button $w.plus -text "+" -padx 1 -pady 0 -font \*symbol\* 
    entry $w.entry -width 6 -textvariable "pageInfo($var)"
    button $w.minus -text "-" -padx 1 -pady 0 -font \*symbol\*
    label $w.units -text "in"
    bind $w.plus <ButtonPress-1>  { StartChange %W 0.1}
    bind $w.plus <ButtonRelease-1> { EndChange %W }
    bind $w.minus <ButtonPress-1>  { StartChange %W -0.1}
    bind $w.minus <ButtonRelease-1> { EndChange %W }
    table $w \
	0,1 $w.label \
	1,1 $w.entry -rspan 2 -fill y \
	1,0 $w.minus -padx 2 -pady 2 \
	2,0 $w.plus -padx 2 -pady { 0 2 } \
	1,2 $w.units -rspan 2 -fill y 
    
}


proc SizeDialog { graph title } {
    global pageInfo
    set top .plotSize
    if { [winfo exists $top] } {
	return
    }
    toplevel $top
    label $top.title -text $title
    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.ok -text "Ok" -command "ApplyPs; destroy $top"
    MakeSizeAdjustor $top.plotWidth "Width" -width
    MakeSizeAdjustor $top.plotHeight "Height" -height
    table $top \
	0,0 $top.title -cspan 2 \
	1,0 $top.plotWidth \
	1,1 $top.plotHeight \
	2,0 $top.cancel -pady 4 -padx 4 -width 1i \
	2,1 $top.ok -pady 4 -padx 4 -width 1i
    set width [winfo fpixels . $pageInfo(-width)]
    set height [winfo fpixels . $pageInfo(-height)]
    if { $width == 0 } {
	set width [expr ($pageInfo(xMax) - $pageInfo(xMin)) / $pageInfo(scale)]
	set pageInfo(-width) [ConvertUnits $width]
    }
    if { $height == 0 } {
	set height [expr ($pageInfo(yMax) - $pageInfo(yMin)) / $pageInfo(scale)]
	set pageInfo(-height) [ConvertUnits $height]
    }
    set pageInfo(-maxpect) 0
}

proc SetPlotSize { } {
    global pageInfo
    set graph $pageInfo(graph)
    switch $pageInfo(plotSize) {
	default { 
	    set pageInfo(-width) 0
	    set pageInfo(-height) 0 
	    set pageInfo(-maxpect) 0
	    set pageInfo(-padx) $pageInfo(oldPadX)
	    set pageInfo(-pady) $pageInfo(oldPadY)
	} maxpect { 
	    set pageInfo(-width) 0
	    set pageInfo(-height) 0
	    set pageInfo(-maxpect) 1
	    set pageInfo(-padx) $pageInfo(oldPadX)
	    set pageInfo(-pady) $pageInfo(oldPadY)
	} resize {
	    set pageInfo(-maxpect) 0
	}
    }
    ApplyPs
}


proc PaperSizeDialog { title } {
    set top .paperSize
    if { [winfo exists $top] } {
	return
    }
    toplevel $top
    label $top.title -text $title
    MakeSizeAdjustor $top.width "Width" -paperwidth
    MakeSizeAdjustor $top.height "Height" -paperheight
    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.ok -text "Ok" -command "ApplyPs; destroy $top"
    table $top \
	0,0 $top.title -cspan 2 \
	1,0 $top.width \
	1,1 $top.height \
	2,0 $top.cancel -pady 4 -padx 4 -width 1i \
	2,1 $top.ok -pady 4 -padx 4 -width 1i
}

proc MarginDialog { graph } {
    set top $graph.top.options
    if { [winfo exists $top] } {
	return
    }
    toplevel $top
    set row 0
    set col 0
    label $top.modeLabel -text "Printer"
    radiobutton $top.color -text "Color" -value "color" \
	-variable pageInfo(-colormode) -command "ApplyPs"
    radiobutton $top.greyscale -text "Greyscale" -value "greyscale" \
	-variable pageInfo(-colormode) -command "ApplyPs"
    table $top \
	$row,$col   $top.modeLabel -anchor e \
	$row,$col+1 $top.color -anchor w \
	$row+1,$col+1 $top.greyscale -anchor w 

    table configure $top r$row -pady { 4 0 }

    label $top.previewLabel -text "Preview"
    radiobutton $top.previewYes -text "Yes" -value "1" \
	-variable pageInfo(-preview) -command "ApplyPs"
    radiobutton $top.previewNo -text "No" -value "0" \
	-variable pageInfo(-preview) -command "ApplyPs"
    set col 2
    table $top \
	$row,$col   $top.previewLabel -anchor e \
	$row,$col+1 $top.previewYes -anchor w \
	$row+1,$col+1 $top.previewNo -anchor w 
    incr row 2

    button $top.cancel -text "Cancel" -command "destroy $top"
    button $top.ok -text "Done" -command "PrintPs $graph $top"
    table $top \
	$row,0 $top.cancel -pady 4 -padx 4 -width 1i \
	$row,1 $top.ok -pady 4 -padx 4 -width 1i
	
}    

proc CenterPlot { } {
    global pageInfo

    set pageInfo(-padx) $pageInfo(oldPadX)
    set pageInfo(-pady) $pageInfo(oldPadY)
    ApplyPs
}


#auto_load Blt_ZoomStack
#if { [catch {source /usr/lib/blt2.4/bltGraph.tcl}] != 0 } {
#  if { [catch {source /usr/local/lib/blt2.4/bltGraph.tcl}] != 0 } {
#     puts "Cannot source Graph functions"
#     exit
#  }
#}                                                                            

set blt_graph_file [ file join $blt_library "bltGraph.tcl" ]

if { [ catch { source $blt_graph_file } ] != 0 } {
     puts "Cannot source Graph functions"
     exit
}

set init true
set xlabel {}
set title ""
set graph .g1
#set tcl_precision 2
stripchart $graph -width 10i -height 8i -title $title
pack $graph
Blt_ZoomStack $graph
#Blt_Crosshairs $graph
    $graph crosshairs on
    bind bltCrosshairs <Any-Motion>   {
        %W crosshairs configure -position @%x,%y
        set coords [$graph invtransform %x %y]
        set x_cursor [lindex $coords 0]
        set y_cursor [lindex $coords 1]
        set xc [string first {.} $x_cursor]
        if { $xc > 0 } {
          incr xc 3
          set x_cursor [string range $x_cursor 0 $xc]
        }
        set xc [string first {.} $y_cursor]
        if { $xc > 0 } {
          incr xc 3
          set y_cursor [string range $y_cursor 0 $xc]
        }
#        puts $coords
    }
    $graph crosshairs configure -color red
    blt::AddBindTag $graph bltCrosshairs

#Blt_ActiveLegend $graph
#Blt_ClosestPoint $graph

htext .footer -text {%%
   if { $onexit == 0 } {
    button $htext(widget).quit -text { quit } -command { exit }
    $htext(widget) append $htext(widget).quit
   }
    button $htext(widget).plot -text { print } -command { 
      PsDialog $graph }
    $htext(widget) append $htext(widget).plot
    frame $htext(widget).cursor
    label $htext(widget).cursor.x_lab -text "X: "
    label $htext(widget).cursor.x -textvariable x_cursor
    label $htext(widget).cursor.y_lab -text "Y: "
    label $htext(widget).cursor.y -textvariable y_cursor
    grid $htext(widget).cursor.x_lab $htext(widget).cursor.x $htext(widget).cursor.y_lab  $htext(widget).cursor.y
    $htext(widget) append $htext(widget).cursor
%% }


table . \
    $graph 1,0 -fill both  \
    .footer 2,0 -fill x -padx 4 -pady 4

table configure . r0 r2 -resize none


fileevent stdin readable ReadStdin
